"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ColorPicker = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const React = __importStar(require("react"));
const tinycolor2_1 = __importDefault(require("tinycolor2"));
const common_1 = require("react-color/lib/components/common");
const ChromePointer_1 = __importDefault(require("react-color/lib/components/chrome/ChromePointer"));
const ChromePointerCircle_1 = __importDefault(require("react-color/lib/components/chrome/ChromePointerCircle"));
const Cancel_1 = __importDefault(require("@material-ui/icons/Cancel"));
const CheckCircle_1 = __importDefault(require("@material-ui/icons/CheckCircle"));
const styles_1 = require("../../styles");
const Column_1 = __importDefault(require("../../layout/Column"));
const Row_1 = __importDefault(require("../../layout/Row"));
const Spacer_1 = __importDefault(require("../../layout/Spacer"));
const v2_1 = __importDefault(require("../ActionBar/v2"));
const InputLabel_1 = __importDefault(require("../InputLabel"));
const TextField_1 = __importDefault(require("../TextField"));
const NumberField_1 = __importDefault(require("../NumberField"));
const ColorPicker = ({ value, onChange, onClose }) => {
    const classes = useStyles();
    // Refs
    const originalColor = React.useRef((0, tinycolor2_1.default)(value));
    // State
    const [hex, setHex] = React.useState(originalColor.current.toHex());
    const [hsl, setHsl] = React.useState(originalColor.current.toHsl());
    const [hsv, setHsv] = React.useState(originalColor.current.toHsv());
    const [opacity, setOpacity] = React.useState(originalColor.current.getAlpha() * 100);
    // Memoizers
    const updatedColor = React.useMemo(() => {
        const color = (0, tinycolor2_1.default)(hex);
        color.setAlpha(opacity / 100);
        return color;
    }, [hex, opacity]);
    const isDirty = React.useMemo(() => {
        return (updatedColor.toHex() !== originalColor.current.toHex() ||
            updatedColor.getAlpha() !== originalColor.current.getAlpha());
    }, [updatedColor]);
    // Callbacks
    const handleColorChange = React.useCallback((data) => {
        const color = (0, tinycolor2_1.default)(data);
        const updatedHsl = color.toHsl();
        const updatedHsv = color.toHsv();
        const updatedHex = color.toHex();
        if (typeof data === 'string') {
            // Hex text field was updated, always save current value in state.
            setHex(data);
            // Don't update the sliders if the color isn't valid.
            if (color.isValid()) {
                setHsl(updatedHsl);
                setHsv(updatedHsv);
            }
        }
        else {
            // The sliders were updated, reset the hue value from data to fix an issue
            // where the hue slider doesn't update when the color is #ffffff.
            updatedHsl.h = data.h;
            updatedHsv.h = data.h;
            setHsl(updatedHsl);
            setHsv(updatedHsv);
            setHex(updatedHex);
        }
    }, []);
    const handleCancel = React.useCallback(() => {
        setHex(originalColor.current.toHex());
        setHsl(originalColor.current.toHsl());
        setHsv(originalColor.current.toHsv());
        setOpacity(originalColor.current.getAlpha() * 100);
        onClose();
    }, [onClose]);
    // Effects
    // Trigger onChange handler when hex or opacity changes.
    React.useEffect(() => {
        if (updatedColor.isValid()) {
            if (updatedColor.getAlpha() < 1) {
                onChange(updatedColor.toRgbString());
            }
            else {
                onChange(updatedColor.toHexString());
            }
        }
    }, [updatedColor, opacity, onChange]);
    // Render
    return ((0, jsx_runtime_1.jsxs)("div", Object.assign({ className: classes.root }, { children: [(0, jsx_runtime_1.jsxs)(Column_1.default, { children: [(0, jsx_runtime_1.jsx)("div", Object.assign({ className: classes.saturation }, { children: (0, jsx_runtime_1.jsx)(common_1.Saturation, { hsl: hsl, hsv: hsv, pointer: ChromePointerCircle_1.default, onChange: handleColorChange }, void 0) }), void 0), (0, jsx_runtime_1.jsxs)("div", { children: [(0, jsx_runtime_1.jsx)(InputLabel_1.default, { children: "Hue" }, void 0), (0, jsx_runtime_1.jsx)("div", Object.assign({ className: classes.slider }, { children: (0, jsx_runtime_1.jsx)(common_1.Hue, { hsl: hsl, hsv: hsv, pointer: ChromePointer_1.default, onChange: handleColorChange }, void 0) }), void 0)] }, void 0), (0, jsx_runtime_1.jsx)(TextField_1.default, { label: "Hex", value: hex, onChange: handleColorChange }, void 0), (0, jsx_runtime_1.jsx)(NumberField_1.default, { label: "Opacity", value: opacity, min: 0, max: 100, onChange: setOpacity }, void 0)] }, void 0), (0, jsx_runtime_1.jsxs)(Row_1.default, Object.assign({ className: classes.actions }, { children: [(0, jsx_runtime_1.jsx)(v2_1.default.Action, { icon: (0, jsx_runtime_1.jsx)(Cancel_1.default, {}, void 0), color: "error", label: "Cancel", onClick: handleCancel }, void 0), (0, jsx_runtime_1.jsx)(Spacer_1.default, {}, void 0), (0, jsx_runtime_1.jsx)(v2_1.default.Action, { icon: (0, jsx_runtime_1.jsx)(CheckCircle_1.default, {}, void 0), color: "success", label: "Done", disabled: !isDirty, onClick: onClose }, void 0)] }), void 0)] }), void 0));
};
exports.ColorPicker = ColorPicker;
const useStyles = (0, styles_1.makeStyles)((theme) => (0, styles_1.createStyles)({
    root: {
        padding: theme.spacing(3),
        maxWidth: 240,
    },
    saturation: {
        position: 'relative',
        width: '100%',
        height: 100,
    },
    slider: {
        position: 'relative',
        width: '100%',
        height: 10,
    },
    actions: {
        marginTop: theme.spacing(3),
        paddingTop: theme.spacing(3),
        borderTop: `1px solid ${theme.actionBar.border}`,
    },
}));
exports.default = exports.ColorPicker;
//# sourceMappingURL=ColorPicker.js.map