"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const jsx_runtime_1 = require("react/jsx-runtime");
const React = __importStar(require("react"));
const immutable = __importStar(require("object-path-immutable"));
const Column_1 = __importDefault(require("../../layout/Column"));
const TextField_1 = __importDefault(require("../../core/TextField"));
const NumberField_1 = __importDefault(require("../../core/NumberField"));
const PresentationFormHelperText_1 = __importDefault(require("./PresentationFormHelperText"));
const getLocalUploads_1 = __importDefault(require("./getLocalUploads"));
const utilities_1 = require("./utilities");
// Inputs
const ArrayInput_1 = __importDefault(require("./inputs/ArrayInput"));
const BooleanInput_1 = __importDefault(require("./inputs/BooleanInput"));
const DateInput_1 = __importDefault(require("./inputs/DateInput"));
const FacebookAuthInput_1 = __importDefault(require("./inputs/FacebookAuthInput"));
const FileInput_1 = __importDefault(require("./inputs/FileInput"));
const GoogleAuthInput_1 = __importDefault(require("./inputs/GoogleAuthInput"));
const ImagePickerFieldInput_1 = __importDefault(require("./inputs/ImagePickerFieldInput"));
const ModalButton_1 = __importDefault(require("./inputs/ModalButton"));
const NumberInput_1 = __importDefault(require("./inputs/NumberInput"));
const OAuthInput_1 = __importDefault(require("./inputs/OAuthInput"));
const OneDriveAuthInput_1 = __importDefault(require("./inputs/OneDriveAuthInput"));
const PlaylistInput_1 = __importDefault(require("./inputs/PlaylistInput"));
const PlaylistInputLegacy_1 = __importDefault(require("./inputs/PlaylistInput/PlaylistInputLegacy"));
const PosterMyWallAuthInput_1 = __importDefault(require("./inputs/PosterMyWallAuthInput"));
const SelectionInput_1 = __importDefault(require("./inputs/SelectionInput"));
const SoundZoneInput_1 = __importDefault(require("./inputs/SoundZoneInput"));
const StringInput_1 = __importDefault(require("./inputs/StringInput"));
const TextInput_1 = __importDefault(require("./inputs/TextInput"));
const ThemeInput_1 = __importDefault(require("./inputs/ThemeInput"));
const ThemeInputLegacy_1 = __importDefault(require("./inputs/ThemeInput/ThemeInputLegacy"));
const ToggleButtonGroupInput_1 = __importDefault(require("./inputs/ToggleButtonGroupInput"));
// These are "fake" presentation properties for name and duration so they can
// be treated the same as application variables when updating the presentation state.
const namePath = ['name'];
const durationPath = ['duration'];
const nameProp = { type: 'string', name: 'name' };
const durationProp = { type: 'number', name: 'duration' };
const PresentationForm = ({ className, presentation, appVersion, errors, themes, soundZones, playlists, selectedPlaylistPath, builderState, newThemeDisabled, newThemeDisabledMessage, onChange, onBuilderStateChange, onPlaylistEdit, onPlaylistCreate, onPlaylistSelect, onThemeEdit, onThemeManage, onThemeAdd, themeOptions, }) => {
    // Refs
    const fileBlobsRef = React.useRef({});
    // Callbacks
    const updatePresentation = React.useCallback((path, value, property, file) => {
        // Remove the value if null or undefined for array inputs.
        const shouldDelete = property.type === 'array' && (value === null || value === undefined);
        const updatedPresentation = shouldDelete
            ? immutable.del(presentation, path)
            : immutable.set(presentation, path, value);
        if (file && value) {
            fileBlobsRef.current[value.url] = file;
        }
        onChange(updatedPresentation, (0, getLocalUploads_1.default)(updatedPresentation, appVersion, fileBlobsRef.current));
    }, [presentation, appVersion, onChange]);
    const setInputState = React.useCallback((path, state) => {
        if (!onBuilderStateChange)
            return;
        // @ts-ignore
        const inputFound = builderState.inputs.find((i) => (0, utilities_1.isPathEqual)(i.path, path));
        const updatedInputs = inputFound
            ? // @ts-ignore
                builderState.inputs.map((i) => (0, utilities_1.isPathEqual)(i.path, path)
                    ? { path, state: Object.assign(Object.assign({}, inputFound.state), state) }
                    : i)
            : // @ts-ignore
                [...builderState.inputs, { path, state }];
        onBuilderStateChange(Object.assign(Object.assign({}, builderState), { inputs: updatedInputs }));
    }, [onBuilderStateChange, builderState]);
    const getInputStateAtPath = React.useCallback((path) => {
        // @ts-ignore
        const input = builderState.inputs.find((i) => {
            return (0, utilities_1.isPathEqual)(path, i.path);
        });
        return input && input.state;
    }, [builderState]);
    // Effects
    // Check hash params for updated properties.
    React.useEffect(() => {
        // Get the window hash without the leading '#'.
        const hash = window.location.hash.substr(1);
        // Parse hash params by splitting on & (or #) and then =.
        //   ie. #applicationVariables.headingText=text => [['applicationVariables.headingText', 'SomeText'], ...]
        const hashParams = hash
            .split(/[&#]/)
            .map((paramStr) => paramStr.split('='));
        hashParams.forEach((hashParam) => {
            if (hashParam.length !== 2)
                return;
            const path = hashParam[0]
                .split('.')
                .filter((pathPart) => !!pathPart);
            if (!path.length)
                return;
            // Try to find the property at this path.
            let property;
            if (path[0] === 'applicationVariables') {
                property = (0, utilities_1.getPropertyAtPath)(appVersion.presentationProperties, path);
            }
            else if ((0, utilities_1.isPathEqual)(path, namePath)) {
                property = nameProp;
            }
            else if ((0, utilities_1.isPathEqual)(path, durationPath)) {
                property = durationProp;
            }
            // @ts-ignore
            if (!property)
                return;
            let value = decodeURIComponent(hashParam[1]);
            try {
                value = JSON.parse(value);
            }
            catch (err) {
                // If we can't parse valid JSON, assume a string value is passed in.
                // This allows the hash param to omit quotes (ie. #headingText=text)
            }
            if (value !== undefined) {
                updatePresentation(path, value, property, undefined);
            }
        });
        // Clear hash parameters to avoid leaking tokens and other sensitive
        // values to the user.
        window.location.hash = '';
    }, [appVersion, updatePresentation]);
    // Render
    const nameError = (0, utilities_1.getErrorAtPath)(errors, namePath);
    const durationError = (0, utilities_1.getErrorAtPath)(errors, durationPath);
    const renderApplicationVariables = (appVars, properties, path, strings) => {
        const propertyTypeIndexes = {};
        const formInputs = properties.map((property) => {
            const propertyPath = [...path, property.name];
            const propertyTypeIndex = propertyTypeIndexes[property.type] || 0;
            propertyTypeIndexes[property.type] = propertyTypeIndex + 1;
            return renderInput(property, propertyTypeIndex, 
            // appVars can be undefined for newly added array items.
            appVars && appVars[property.name], appVars, propertyPath, strings);
        });
        return formInputs;
    };
    const renderInput = (property, propertyTypeIndex, value = property.default, parentValue, path, strings) => {
        const key = property.name;
        const label = strings[property.name] || property.name;
        const constraints = property.constraints || {};
        const isDisabled = !!property.disable;
        const inputError = (0, utilities_1.getErrorAtPath)(errors, path);
        const hasError = !!inputError;
        // Render the component inline by calling the function instead of using JSX.
        // This causes `null` to be returned instead of an empty <PresentationFormHelperText />
        // element. Some inputs (ie. PlaylistInput) provide their own default helperText if not set
        // for the input and rely `null` being returned here.
        const helperText = (0, PresentationFormHelperText_1.default)({
            property,
            path,
            strings,
            error: inputError,
        });
        if (property.hide) {
            return null;
        }
        switch (property.type) {
            case 'string': {
                return ((0, jsx_runtime_1.jsx)(StringInput_1.default, { label: label, value: value, helperText: helperText, error: hasError, disabled: isDisabled, constraints: constraints, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'text': {
                return ((0, jsx_runtime_1.jsx)(TextInput_1.default, { label: label, value: value, helperText: helperText, error: hasError, disabled: isDisabled, constraints: constraints, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'number': {
                return ((0, jsx_runtime_1.jsx)(NumberInput_1.default, { label: label, value: value, defaultValue: property.default, helperText: helperText, error: hasError, disabled: isDisabled, constraints: constraints, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'boolean': {
                return ((0, jsx_runtime_1.jsx)(BooleanInput_1.default, { label: label, value: value, helperText: helperText, error: hasError, disabled: isDisabled, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'selection': {
                return ((0, jsx_runtime_1.jsx)(SelectionInput_1.default, { label: label, value: value, parentValue: parentValue, multiple: property.multiple, searchable: property.searchable, selectable: property.selectable, sortable: property.sortable, options: property.options, optionsUrl: property.options_url, helperText: helperText, error: hasError, disabled: isDisabled, onChange: (newValue) => updatePresentation(path, newValue, property), strings: strings, inputState: getInputStateAtPath(path), onInputStateChange: (state) => {
                        setInputState(path, state);
                    } }, key));
            }
            case 'toggleButtonGroup': {
                return ((0, jsx_runtime_1.jsx)(ToggleButtonGroupInput_1.default, { label: label, value: value, options: property.options, helperText: helperText, disabled: isDisabled, onChange: (newValue) => {
                        // Enforce at least one button to be active
                        if (newValue !== null) {
                            updatePresentation(path, newValue, property);
                        }
                    }, exclusive: property.exclusive, strings: strings }, key));
            }
            case 'selectionWithImages': {
                return ((0, jsx_runtime_1.jsx)(ImagePickerFieldInput_1.default, { parentValue: parentValue, value: value, imagesUrl: property.images_url, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'date': {
                return ((0, jsx_runtime_1.jsx)(DateInput_1.default, { label: label, value: value, helperText: helperText, error: hasError, disabled: isDisabled, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'file': {
                return ((0, jsx_runtime_1.jsx)(FileInput_1.default, { label: label, value: value, helperText: helperText, error: hasError, disabled: isDisabled, constraints: constraints, onChange: (newValue, file) => 
                    // @ts-ignore
                    updatePresentation(path, newValue, property, file) }, key));
            }
            case 'oAuth': {
                return ((0, jsx_runtime_1.jsx)(OAuthInput_1.default, { path: path, label: label, value: value, provider: property.provider, authUrl: property.auth_url || '', verifyUrl: property.verify_url || '', verifyQsParam: property.verify_qs_param || '', helperText: helperText, error: hasError, disabled: isDisabled, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'facebookAuth': {
                return (
                // @ts-ignore
                (0, jsx_runtime_1.jsx)(FacebookAuthInput_1.default, { path: path, label: label, value: value, authUrl: property.auth_url, verifyUrl: property.verify_url, verifyQsParam: property.verify_qs_param, helperText: helperText, error: hasError, disabled: isDisabled, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'googleAuth': {
                return (
                // @ts-ignore
                (0, jsx_runtime_1.jsx)(GoogleAuthInput_1.default, { path: path, label: label, value: value, authUrl: property.auth_url, verifyUrl: property.verify_url, verifyQsParam: property.verify_qs_param, helperText: helperText, error: hasError, disabled: isDisabled, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'onedriveAuth': {
                return (
                // @ts-ignore
                (0, jsx_runtime_1.jsx)(OneDriveAuthInput_1.default, { path: path, label: label, value: value, authUrl: property.auth_url, verifyUrl: property.verify_url, verifyQsParam: property.verify_qs_param, helperText: helperText, error: hasError, disabled: isDisabled, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'postermywallAuth': {
                return (
                // @ts-ignore
                (0, jsx_runtime_1.jsx)(PosterMyWallAuthInput_1.default, { path: path, label: label, value: value, authUrl: property.auth_url, verifyUrl: property.verify_url, verifyQsParam: property.verify_qs_param, helperText: helperText, error: hasError, disabled: isDisabled, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'theme': {
                if (!themeOptions) {
                    // This is to only here to support the RaydiantKit Simulator
                    return ((0, jsx_runtime_1.jsx)(ThemeInputLegacy_1.default, { label: label, 
                        // @ts-ignore
                        value: presentation.themeId, 
                        // @ts-ignore
                        themes: themes, helperText: helperText, error: hasError, disabled: isDisabled, onChange: (newValue) => updatePresentation(['themeId'], newValue, property) }, key));
                }
                return ((0, jsx_runtime_1.jsx)(ThemeInput_1.default, { label: label, 
                    // @ts-ignore
                    value: presentation.themeId, themeOptions: themeOptions, helperText: helperText, error: hasError, disabled: isDisabled, addDisabled: newThemeDisabled, addDisabledMessage: newThemeDisabledMessage, onChange: (newValue) => updatePresentation(['themeId'], newValue, property), onEdit: onThemeEdit, onManage: onThemeManage, 
                    // @ts-ignore
                    onAdd: onThemeAdd }, key));
            }
            case 'playlist': {
                if (!onPlaylistEdit) {
                    // If onPlaylistEdit is not provided then use the legacy playlist input.
                    // This is to only here to support the RaydiantKit Simulator
                    return ((0, jsx_runtime_1.jsx)(PlaylistInputLegacy_1.default, { label: label, value: value, 
                        // @ts-ignore
                        playlists: playlists, helperText: helperText, error: hasError, disabled: isDisabled, propertyTypeIndex: propertyTypeIndex, onChange: (newValue) => updatePresentation(path, newValue, property), 
                        // @ts-ignore
                        onEdit: (playlistId) => onPlaylistEdit(playlistId, path), 
                        // @ts-ignore
                        onCreate: () => onPlaylistCreate(path) }, key));
                }
                return ((0, jsx_runtime_1.jsx)(PlaylistInput_1.default, { label: label, value: value, path: path, 
                    // @ts-ignore
                    playlists: playlists, helperText: helperText, error: hasError, disabled: isDisabled, propertyTypeIndex: propertyTypeIndex, 
                    // @ts-ignore
                    selectedPlaylistPath: selectedPlaylistPath, onChange: (newValue) => updatePresentation(path, newValue, property), onEdit: (playlistId) => onPlaylistEdit(playlistId, path), 
                    // @ts-ignore
                    onSelect: () => onPlaylistSelect(path) }, key));
            }
            case 'soundZone': {
                return ((0, jsx_runtime_1.jsx)(SoundZoneInput_1.default, { label: label, value: value, 
                    // @ts-ignore
                    soundZones: soundZones, helperText: helperText, error: hasError, disabled: isDisabled, onChange: (newValue) => updatePresentation(path, newValue, property) }, key));
            }
            case 'array': {
                const singularLabel = (property.singular_name && strings[property.singular_name]) ||
                    property.singular_name;
                return ((0, jsx_runtime_1.jsx)(ArrayInput_1.default, { path: path, label: label, value: value, 
                    // @ts-ignore
                    singularLabel: singularLabel, 
                    // @ts-ignore
                    properties: property.properties, helperText: helperText, error: hasError, disabled: isDisabled, constraints: constraints, strings: strings, errors: errors, onChange: (newValue) => updatePresentation(path, newValue, property), renderForm: (itemValue, itemProperties, itemPath) => ((0, jsx_runtime_1.jsx)(Column_1.default, { children: renderApplicationVariables(itemValue, itemProperties, [...path, ...itemPath], strings) }, void 0)), onInputStateChange: (state) => {
                        setInputState(path, state);
                    } }, key));
            }
            case 'modal': {
                return ((0, jsx_runtime_1.jsx)(ModalButton_1.default, { label: label, sourceUrl: property.sourceUrl, backgroundColor: property.backgroundColor, hoveredBackgroundColor: property.hoveredBackgroundColor, textColor: property.textColor, parentValue: parentValue, helperText: helperText, disabled: isDisabled, onChange: (newValue) => 
                    // TODO: This will save the state to the db, we used to use updateIgnoredApplicationVariables
                    // to bypass the db but this seems hacky, find a better way.
                    // updateIgnoredApplicationVariables(path, newValue)
                    updatePresentation(path, newValue, property) }, key));
            }
            default:
                return null;
        }
    };
    return ((0, jsx_runtime_1.jsxs)(Column_1.default, Object.assign({ className: className }, { children: [(0, jsx_runtime_1.jsx)(TextField_1.default, { label: "name", value: presentation.name, onChange: (name) => updatePresentation(namePath, name, nameProp), helperText: (0, jsx_runtime_1.jsx)(PresentationFormHelperText_1.default, { property: nameProp, path: namePath, error: nameError, strings: appVersion.strings }, void 0), error: !!nameError }, void 0), renderApplicationVariables(presentation.applicationVariables, appVersion.presentationProperties, ['applicationVariables'], appVersion.strings), appVersion.configurableDuration && ((0, jsx_runtime_1.jsx)(NumberField_1.default, { label: "Duration", value: presentation.duration, onChange: (duration) => updatePresentation(durationPath, duration, durationProp), helperText: (0, jsx_runtime_1.jsx)(PresentationFormHelperText_1.default, { property: durationProp, path: durationPath, error: durationError, strings: appVersion.strings }, void 0), error: !!durationError }, void 0))] }), void 0));
};
exports.default = PresentationForm;
//# sourceMappingURL=PresentationForm.js.map