"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const jsx_runtime_1 = require("react/jsx-runtime");
const React = __importStar(require("react"));
const classnames_1 = __importDefault(require("classnames"));
const PlaylistEdit_1 = __importDefault(require("../../../../icons/PlaylistEdit"));
const OpenLibrary_1 = __importDefault(require("../../../../icons/OpenLibrary"));
const InputLabel_1 = __importDefault(require("../../../../core/InputLabel"));
const Row_1 = __importDefault(require("../../../../layout/Row"));
const styles_1 = require("../../../../styles");
const mixins_1 = require("../../../../mixins");
const utilities_1 = require("../../utilities");
const useStyles = (0, styles_1.makeStyles)((theme) => {
    const lightNavy = '#225887';
    return (0, styles_1.createStyles)({
        selected: {
            boxShadow: `0px 0px 0px 3px ${lightNavy}`,
            '& $selectPlaylist': {
                color: theme.palette.primary.main,
            },
        },
        actions: {
            alignItems: 'flex-start',
            padding: theme.spacing(0.5, 1),
            marginLeft: theme.spacing(-1),
            marginRight: theme.spacing(-1),
            width: `calc(100% + ${theme.spacing(2)}px)`,
            borderRadius: theme.borderRadius.sm,
            fontSize: 24,
            [theme.breakpoints.down('xs')]: {
                padding: theme.spacing(2, 1),
            },
        },
        action: Object.assign(Object.assign({}, (0, mixins_1.buttonReset)()), { position: 'relative', overflow: 'hidden', display: 'flex', color: theme.palette.text.secondary, padding: theme.spacing(1), '&:hover': {
                color: theme.palette.primary.main,
            }, '&:disabled': {
                cursor: 'not-allowed',
                color: 'rgba(0, 0, 0, 0.2)',
                '&:hover': {
                    color: 'rgba(0, 0, 0, 0.2)',
                },
            } }),
        iconWithLabel: {
            display: 'flex',
            flexDirection: 'column',
            alignItems: 'center',
            '& > span': {
                marginTop: theme.spacing(0.25),
                fontSize: theme.fontSizes.xxs,
                letterSpacing: 0.06,
            },
        },
        publishLock: {
            fontSize: 24, // Sets settings icon size
        },
        editPlaylist: {
            flex: 1,
        },
        selectPlaylist: {},
        editPlaylistIcon: {
            // Fix size / alignment.
            transform: 'scale(1.25)',
        },
        playlist: Object.assign(Object.assign({}, (0, mixins_1.textTruncate)()), { marginLeft: theme.spacing(2), fontSize: theme.fontSizes.xl, fontWeight: 300, lineHeight: 1.09, letterSpacing: 0.25, [theme.breakpoints.down('xs')]: {
                fontSize: theme.fontSizes.lg,
            } }),
        playlistLabel: {
            fontSize: theme.fontSizes.sm,
            color: theme.palette.text.secondary,
            marginTop: theme.spacing(0.5),
        },
    });
});
const PlaylistInput = ({ label, value, path, playlists, selectedPlaylistPath, error, disabled, helperText, onSelect = () => { }, onChange, onEdit = () => { }, }) => {
    const classes = useStyles();
    const isSelected = (0, utilities_1.isPathEqual)(selectedPlaylistPath, path);
    const playlist = playlists.find((pl) => pl.id === value);
    const isDeleted = playlist && !!playlist.resource.deletedAt;
    // Assume the playlist was assigned by an admin if there is a playlist id but we can't
    // find it in the current user's playlists. This is a workaround, ideally the API would return
    // the basic information about the assigned playlist if the current user doesn't have access to it.
    const isAdminAssigned = value && !playlist;
    // It would be better to use the same permission logic in the Dashboard to determine but want to avoid
    // the duplicating permission functions in elements. For now, assume that the user does not have access
    // to edit this playlist if it is not provided.
    const isEditable = !isDeleted && !isAdminAssigned;
    const isSelectable = !isAdminAssigned;
    const handlePlaylistSelect = React.useCallback(() => __awaiter(void 0, void 0, void 0, function* () {
        // We still want the playlist input to call onChange in order to update the preview. To do this,
        // onSelect returns a promise that will resolve to the selected playlist id or empty if the user
        // did not select anything.
        const playlistId = yield onSelect();
        // @ts-ignore
        if (playlistId) {
            onChange(playlistId);
        }
    }), [onChange, onSelect]);
    const handlePlaylistClick = React.useCallback(() => {
        if (playlist && isEditable) {
            onEdit(value);
        }
        else if (!playlist && isEditable) {
            handlePlaylistSelect();
        }
    }, [playlist, isEditable, onEdit, value, handlePlaylistSelect]);
    let playlistLabel = '';
    if (playlist && !isDeleted) {
        playlistLabel = playlist.name;
    }
    else if (isAdminAssigned) {
        playlistLabel = 'Administrator assigned playlist';
    }
    else {
        playlistLabel = 'Assign a playlist';
    }
    return ((0, jsx_runtime_1.jsxs)("div", { children: [(0, jsx_runtime_1.jsx)(InputLabel_1.default, Object.assign({ error: error, disabled: disabled }, { children: label }), void 0), (0, jsx_runtime_1.jsxs)(Row_1.default, Object.assign({ className: (0, classnames_1.default)(classes.actions, isSelected && classes.selected) }, { children: [(0, jsx_runtime_1.jsxs)("button", Object.assign({ className: (0, classnames_1.default)(classes.action, classes.editPlaylist), disabled: !isEditable, onClick: handlePlaylistClick }, { children: [(0, jsx_runtime_1.jsxs)("div", Object.assign({ className: classes.iconWithLabel }, { children: [(0, jsx_runtime_1.jsx)(PlaylistEdit_1.default, { fontSize: "inherit", className: (0, classnames_1.default)(classes.editPlaylistIcon) }, void 0), (0, jsx_runtime_1.jsx)("span", { children: "edit" }, void 0)] }), void 0), (0, jsx_runtime_1.jsxs)("div", Object.assign({ className: classes.playlist }, { children: [playlistLabel, (0, jsx_runtime_1.jsx)("div", Object.assign({ className: classes.playlistLabel }, { children: helperText || `${label}'s playlist` }), void 0)] }), void 0)] }), void 0), (0, jsx_runtime_1.jsx)("button", Object.assign({ className: (0, classnames_1.default)(classes.action, classes.selectPlaylist, 'tour-device-playlist-select'), disabled: !isSelectable, onClick: handlePlaylistSelect }, { children: (0, jsx_runtime_1.jsxs)("div", Object.assign({ className: classes.iconWithLabel }, { children: [(0, jsx_runtime_1.jsx)(OpenLibrary_1.default, { fontSize: "inherit" }, void 0), (0, jsx_runtime_1.jsx)("span", { children: "playlists" }, void 0)] }), void 0) }), void 0)] }), void 0)] }, void 0));
};
exports.default = PlaylistInput;
//# sourceMappingURL=PlaylistInput.js.map