"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const jsx_runtime_1 = require("react/jsx-runtime");
const React = __importStar(require("react"));
const lodash_isequal_1 = __importDefault(require("lodash.isequal"));
const Link_1 = __importDefault(require("../../../../core/Link"));
const MultiSelectField_1 = __importDefault(require("../../../../core/MultiSelectField"));
const SelectField_1 = __importDefault(require("../../../../core/SelectField"));
const replacePropNameWithValue_1 = __importDefault(require("../../../../helpers/replacePropNameWithValue"));
class SelectionInput extends React.Component {
    constructor() {
        super(...arguments);
        this.state = {
            options: null,
            optionsError: '',
        };
    }
    componentDidMount() {
        const { optionsUrl, parentValue } = this.props;
        if (optionsUrl) {
            const dependentProps = this.getDependentProps();
            // All dependencies must be not be undefined in order to fetch options.
            const shouldFetchOptions = dependentProps.every((propName) => parentValue[propName] !== undefined);
            if (shouldFetchOptions) {
                this.fetchOptions();
            }
        }
    }
    componentDidUpdate(prevProps) {
        const { optionsUrl, parentValue } = this.props;
        if (optionsUrl) {
            // Re-fetch options when one of the properties that the options url
            // depends on has changed.
            const updatedDependentProps = this.getUpdatedDependentProps(prevProps.parentValue);
            // All updated dependencies must be not be undefined in order to fetch options.
            const updatedDependentPropsChanged = updatedDependentProps.length > 0 &&
                updatedDependentProps.every((propName) => parentValue[propName] !== undefined);
            const optionsUrlChanged = prevProps.optionsUrl !== optionsUrl;
            const shouldFetchOptions = updatedDependentPropsChanged || optionsUrlChanged;
            if (shouldFetchOptions) {
                // When parent value is changed, set the value of current element back to default
                this.fetchOptions();
            }
        }
    }
    getDependentProps() {
        const { optionsUrl = '' } = this.props;
        // Find the prop names that the URL depends on.
        const dependsOn = [];
        optionsUrl.replace(/\{\{(.*?)\}\}/g, (token, propName) => {
            dependsOn.push(propName);
            // We need to return something to appease TS but we don't actually
            // use the result of the replace.
            return token;
        });
        return dependsOn;
    }
    getUpdatedDependentProps(prevParentValue) {
        const { parentValue } = this.props;
        return this.getDependentProps().filter((propName) => !(0, lodash_isequal_1.default)(parentValue[propName], prevParentValue[propName]));
    }
    fetchOptions() {
        return __awaiter(this, void 0, void 0, function* () {
            const { optionsUrl = '', parentValue } = this.props;
            const defaultErrorMessage = 'Failed to load options.';
            const url = (0, replacePropNameWithValue_1.default)(optionsUrl, parentValue, true);
            const resp = yield fetch(url);
            if (resp.ok) {
                try {
                    const options = yield resp.json();
                    // Set the default value before the options. This fixes an issue with
                    // multi-selects not ordering the selected items to the top of the list.
                    this.checkDefaultOptions(options);
                    this.setState({ options });
                }
                catch (err) {
                    this.setState({ optionsError: defaultErrorMessage });
                }
            }
            else {
                try {
                    const error = yield resp.json();
                    this.setState({ optionsError: error.message || defaultErrorMessage });
                }
                catch (err) {
                    this.setState({ optionsError: defaultErrorMessage });
                }
            }
        });
    }
    checkDefaultOptions(options) {
        const { value, multiple, onChange } = this.props;
        const isValueUnset = value === null || value === undefined;
        const hasOptions = options.length > 0;
        const isValueInOptions = multiple
            ? Array.isArray(value) &&
                value.every((v) => options.some((opt) => opt.value === v))
            : options.some((opt) => opt.value === value);
        const shouldSetValueToDefault = isValueUnset || !isValueInOptions;
        if (shouldSetValueToDefault && hasOptions) {
            const defaultValue = options
                .filter((opt) => opt.default)
                .map((opt) => opt.value);
            if (defaultValue.length > 0) {
                if (multiple) {
                    onChange(defaultValue);
                }
                else {
                    onChange(defaultValue[0]);
                }
            }
        }
    }
    getLabel(option) {
        const { strings } = this.props;
        return (option.name && strings[option.name]) || option.name || option.label;
    }
    render() {
        const { label, value, multiple, searchable, selectable, onInputStateChange, sortable, helperText, error, disabled, onChange, inputState, } = this.props;
        const options = this.state.options || this.props.options || [];
        if (multiple) {
            // Ensure it's an array to appease TS.
            const multiSelectValue = Array.isArray(value)
                ? value
                : Array.from(value || []);
            const selectedOptionValue = inputState &&
                inputState.selectedOption &&
                inputState.selectedOption.value;
            return ((0, jsx_runtime_1.jsx)(MultiSelectField_1.default, Object.assign({ label: label, value: multiSelectValue, helperText: helperText, error: error, disabled: disabled, searchable: searchable, selectable: selectable, onChange: onChange, sortable: sortable }, { children: options.map((opt, index) => (
                // @ts-ignore
                (0, jsx_runtime_1.jsx)(MultiSelectField_1.default.Option, { value: opt.value, label: this.getLabel(opt), rightLabel: opt.rightLabel, selected: selectable && selectedOptionValue === opt.value, onSelect: selectable &&
                        ((option) => onInputStateChange({ selectedOption: option })) }, index))) }), void 0));
        }
        // Ensure it's not an array to appease TS.
        const singleSelectValue = Array.isArray(value) ? value[0] : value;
        const hasThumbnails = options.some(({ thumbnailUrl }) => !!thumbnailUrl);
        const selectValue = singleSelectValue || (options[0] ? options[0].value : '');
        const chosenOption = selectValue
            ? (options || []).find((option) => option.value === selectValue)
            : undefined;
        const optionHelperText = chosenOption ? chosenOption.helperText : undefined;
        const optionHelperLink = chosenOption ? chosenOption.helperLink : undefined;
        const optionHelper = optionHelperLink ? ((0, jsx_runtime_1.jsx)(Link_1.default, Object.assign({ target: "_blank", href: optionHelperLink }, { children: optionHelperText || optionHelperLink }), void 0)) : (optionHelperText);
        const optionError = chosenOption ? chosenOption.error : false;
        return ((0, jsx_runtime_1.jsx)(SelectField_1.default, Object.assign({ label: label, value: selectValue, onChange: onChange, helperText: optionHelper || helperText, error: optionError || error, disabled: disabled, native: !hasThumbnails }, { children: options.map((opt, index) => hasThumbnails ? ((0, jsx_runtime_1.jsx)(SelectField_1.default.Item, Object.assign({ value: opt.value, thumbnailUrl: opt.thumbnailUrl }, { children: this.getLabel(opt) }), index)) : ((0, jsx_runtime_1.jsx)("option", Object.assign({ value: opt.value }, { children: this.getLabel(opt) }), index))) }), void 0));
    }
}
SelectionInput.defaultProps = {
    multiple: false,
    searchable: false,
    options: [],
    optionsUrl: '',
};
exports.default = SelectionInput;
//# sourceMappingURL=SelectionInput.js.map