"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const jsx_runtime_1 = require("react/jsx-runtime");
const React = __importStar(require("react"));
const classnames_1 = __importDefault(require("classnames"));
const debounce_1 = __importDefault(require("debounce"));
const Tabs_1 = __importDefault(require("../../core/Tabs/Tabs"));
const Hidden_1 = __importDefault(require("../../layout/Hidden"));
const PresentationTypes_1 = require("../PresentationTypes");
const PresentationPreview_styles_1 = __importStar(require("./PresentationPreview.styles"));
const PresentationPreview = ({ appVersion, previewMode, showBorder, children, onPreviewModeChange, }) => {
    const classes = (0, PresentationPreview_styles_1.default)();
    // Refs
    const previewRef = React.useRef(null);
    // State
    const [dimensions, setDimensions] = React.useState({
        width: 0,
        height: 0,
    });
    // Callbacks
    const calculateDimensions = React.useCallback(() => {
        if (!previewRef.current)
            return;
        const rect = previewRef.current.getBoundingClientRect();
        const previewWidth = Math.ceil(rect.width);
        const previewHeight = Math.ceil(rect.height);
        if (previewWidth !== dimensions.width ||
            previewHeight !== dimensions.height) {
            setDimensions({ width: previewWidth, height: previewHeight });
        }
    }, [dimensions, setDimensions]);
    // Effects
    // Calculate initial dimensions
    React.useLayoutEffect(calculateDimensions);
    // Re-calculate on window resize
    React.useEffect(() => {
        const calculateDebounced = (0, debounce_1.default)(calculateDimensions);
        window.addEventListener('resize', calculateDebounced);
        window.addEventListener('orientationchange', calculateDebounced);
        return () => {
            window.removeEventListener('resize', calculateDebounced);
            window.removeEventListener('orientationchange', calculateDebounced);
        };
    }, [calculateDimensions]);
    // Render
    const isLandscape = previewMode !== PresentationTypes_1.PreviewMode.Vertical;
    // To ensure the preview content is 1920x1080 we need to compensate for
    // the border width around the preview.
    const frameSize = showBorder ? PresentationPreview_styles_1.previewBorderWidth * 2 : 0;
    const width = (isLandscape ? 1920 : 1080) + frameSize;
    const height = (isLandscape ? 1080 : 1920) + frameSize;
    const scaleX = dimensions.width / width;
    const scaleY = dimensions.height / height;
    const scale = Math.min(scaleX, scaleY);
    const marginTop = -(height / 2) * scale;
    const marginLeft = -(width / 2) * scale;
    return ((0, jsx_runtime_1.jsxs)("div", Object.assign({ className: classes.root }, { children: [(0, jsx_runtime_1.jsx)("div", Object.assign({ className: classes.main }, { children: (0, jsx_runtime_1.jsx)("div", Object.assign({ ref: previewRef, className: classes.previewContainer }, { children: (0, jsx_runtime_1.jsx)("div", Object.assign({ className: (0, classnames_1.default)(classes.preview, showBorder && classes.previewBorder), style: {
                            width,
                            height,
                            marginTop,
                            marginLeft,
                            transform: `scale(${scale})`,
                        } }, { children: children }), void 0) }), void 0) }), void 0), (0, jsx_runtime_1.jsxs)("div", Object.assign({ className: classes.footer }, { children: [appVersion && ((0, jsx_runtime_1.jsxs)("div", Object.assign({ className: classes.app }, { children: [(0, jsx_runtime_1.jsx)("div", { className: classes.appIcon, style: {
                                    backgroundImage: appVersion.iconUrl
                                        ? `url(${appVersion.iconUrl})`
                                        : '',
                                } }, void 0), appVersion.name && ((0, jsx_runtime_1.jsxs)("div", { children: [(0, jsx_runtime_1.jsx)("div", Object.assign({ className: classes.appName }, { children: appVersion.name }), void 0), (0, jsx_runtime_1.jsx)("div", Object.assign({ className: classes.appDescription }, { children: appVersion.description || appVersion.strings.description }), void 0)] }, void 0))] }), void 0)), !appVersion && onPreviewModeChange && ((0, jsx_runtime_1.jsxs)("div", Object.assign({ className: classes.app }, { children: [(0, jsx_runtime_1.jsx)("div", { className: classes.appIcon, style: { backgroundColor: 'transparent' } }, void 0), (0, jsx_runtime_1.jsxs)("div", { children: [(0, jsx_runtime_1.jsx)("div", { className: classes.appName }, void 0), (0, jsx_runtime_1.jsx)("div", { className: classes.appDescription }, void 0)] }, void 0)] }), void 0)), onPreviewModeChange && ((0, jsx_runtime_1.jsx)(Hidden_1.default, Object.assign({ smDown: true }, { children: (0, jsx_runtime_1.jsxs)(Tabs_1.default, Object.assign({ inline: true }, { children: [(0, jsx_runtime_1.jsx)(Tabs_1.default.Item, { icon: "horizontalScreen", label: "Horizontal", active: previewMode === PresentationTypes_1.PreviewMode.Horizontal, onClick: () => onPreviewModeChange(PresentationTypes_1.PreviewMode.Horizontal) }, void 0), (0, jsx_runtime_1.jsx)(Tabs_1.default.Item, { icon: "verticalScreen", label: "Vertical", active: previewMode === PresentationTypes_1.PreviewMode.Vertical, onClick: () => onPreviewModeChange(PresentationTypes_1.PreviewMode.Vertical) }, void 0)] }), void 0) }), void 0))] }), void 0)] }), void 0));
};
exports.default = PresentationPreview;
//# sourceMappingURL=PresentationPreview.js.map